var mysql = require('mysql');

module.exports = function(options) {

    var db = mysql.createConnection (
        {
            host                : options.host,
            port                : options.port,
            user                : options.user,
            password            : options.password,
            database            : options.database,
            multipleStatements  : true,
            connectTimeout      : 30000
        }
    );

    var STATUS =
    {
        PENDING : 'pending',
        SUCCESS : 'success',
        FAILED  : 'failed'
    };

    function getLastVersionNumber(callback)
    {
        var sql = 'select version from database_script_history where status="success" order by version desc limit 1';

        db.query( {sql: sql, timeout: 30000 }, function(err, result)
        {
            if (err) return callback(err);

            var lastVersion = 0;

            if (result.length > 0) lastVersion = result[0].version;

            callback(null, lastVersion);
        });
    }

    function insertAttemptIntoHistoryAsPending(values, callback)
    {
        var sql = 'insert into database_script_history (version, status, name) values (?, ?, ?)';

        db.query( {sql: sql, timeout: 30000 }, [values.scriptVersion, STATUS.PENDING, values.name], function(err)
        {
            if (err) return callback(err);

            callback(null, values);

        })
    }

    function executeScript(values, callback)
    {
        db.query( {sql: values.scriptContent, timeout: 30000 }, function(err)
        {
            if (err)
            {
                updateHistory( STATUS.FAILED, new Date(), values.scriptVersion, function()
                {
                    console.warn( '| Script version "%d" failed to apply.', values.scriptVersion );
                    callback(err);
                });

            } else
            {
                updateHistory(STATUS.SUCCESS, new Date(), values.scriptVersion, function(err)
                {
                    if (err) return callback(err);

                    console.log('| Script version "%d" was successfully applied to the database.', values.scriptVersion);

                    callback();
                })
            }
        })
    }

    function updateHistory(status, createdAt, version, callback)
    {
        var sql = 'update database_script_history set status=?, createdAt=? where version=? and status="pending"';

        db.query( {sql: sql, timeout: 30000 }, [status, createdAt, version], function(err)
        {
            if (err) return callback(err);

            callback();

        })
    }

    function cleanup()
    {
        db.end( function(err)
        {
            if ( err )
            {
                console.log('                    error during connection close : ' + err);
            }

            db.destroy();
        });
    }

    var service =
    {
        getLastVersionNumber                : getLastVersionNumber,
        insertAttemptIntoHistoryAsPending   : insertAttemptIntoHistoryAsPending,
        executeScript                       : executeScript,
        cleanup                             : cleanup
    };

    return service;

};